searchData={"items":[{"type":"module","title":"TopggEx","doc":"A comprehensive Elixir client for the Top.gg API.\n\nTopggEx provides a complete client for interacting with the Top.gg API,\nallowing you to post bot statistics, retrieve bot information, check votes,\nand handle webhooks with full type safety and excellent documentation.","ref":"TopggEx.html"},{"type":"module","title":"Main Modules - TopggEx","doc":"- `TopggEx.Api` - Client for interacting with the Top.gg REST API\n- `TopggEx.Webhook` - Webhook handler for receiving vote notifications","ref":"TopggEx.html#module-main-modules"},{"type":"module","title":"Quick Start - TopggEx","doc":"","ref":"TopggEx.html#module-quick-start"},{"type":"module","title":"API Client - TopggEx","doc":"# Create a new API client\n    {:ok, api} = TopggEx.Api.new(\"your_topgg_token_here\")\n\n    # Post bot statistics\n    {:ok, _stats} = TopggEx.Api.post_stats(api, %{server_count: 100})\n\n    # Get bot information\n    {:ok, bot} = TopggEx.Api.get_bot(api, \"bot_id\")","ref":"TopggEx.html#module-api-client"},{"type":"module","title":"Webhook Handler - TopggEx","doc":"# In your Phoenix router\n    pipeline :webhook do\n      plug :accepts, [\"json\"]\n      plug TopggEx.Webhook, authorization: \"your_webhook_auth_token\"\n    end\n\n    scope \"/webhooks\" do\n      pipe_through :webhook\n      post \"/topgg\", YourController, :handle_vote\n    end","ref":"TopggEx.html#module-webhook-handler"},{"type":"module","title":"Configuration - TopggEx","doc":"Before using the API client, ensure you have a Finch instance running\nin your application supervision tree:\n\n    children = [\n      {Finch, name: :topgg_finch}\n    ]","ref":"TopggEx.html#module-configuration"},{"type":"module","title":"Links - TopggEx","doc":"- [Top.gg API Documentation](https://docs.top.gg)\n- [Top.gg Webhook Documentation](https://docs.top.gg/resources/webhooks/)","ref":"TopggEx.html#module-links"},{"type":"module","title":"TopggEx.Webhook","doc":"Top.gg Webhook handler for Plug-based HTTP servers.\n\nThis module provides webhook handling functionality for receiving vote notifications\nfrom Top.gg, compatible with Phoenix and other Plug-based HTTP servers.","ref":"TopggEx.Webhook.html"},{"type":"module","title":"Examples - TopggEx.Webhook","doc":"","ref":"TopggEx.Webhook.html#module-examples"},{"type":"module","title":"Using the Functional Listener (Recommended) - TopggEx.Webhook","doc":"# In a Phoenix router:\n    defmodule MyAppWeb.Router do\n      use MyAppWeb, :router\n\n      # Create the webhook handler\n      webhook_handler = TopggEx.Webhook.listener(fn payload, conn ->\n        case payload do\n          %{\"user\" => user_id, \"type\" => \"upvote\", \"bot\" => bot_id} ->\n            # Handle the vote\n            MyApp.handle_user_vote(user_id, bot_id)\n            IO.puts(\"User #{user_id} voted for bot #{bot_id}!\")\n\n          %{\"user\" => user_id, \"type\" => \"test\"} ->\n            # Handle test webhook\n            IO.puts(\"Test webhook from user #{user_id}\")\n        end\n\n        # Response is handled automatically by the listener\n      end, authorization: \"your_webhook_auth_token\")\n\n      scope \"/webhooks\" do\n        pipe_through :api\n        post \"/topgg\", webhook_handler\n      end\n    end\n\n    # In a standalone Plug application:\n    defmodule MyApp.Router do\n      use Plug.Router\n\n      plug :match\n      plug :dispatch\n\n      # Option 1: Use the convenient handle_webhook function (Recommended)\n      post \"/webhook\" do\n        TopggEx.Webhook.handle_webhook(conn, \"your_webhook_auth_token\", fn payload ->\n          case payload do\n            %{\"user\" => user_id, \"type\" => \"upvote\", \"bot\" => bot_id} ->\n              MyApp.handle_user_vote(user_id, bot_id)\n              IO.puts(\"User #{user_id} voted for bot #{bot_id}!\")\n\n            %{\"user\" => user_id, \"type\" => \"test\"} ->\n              IO.puts(\"Test webhook from user #{user_id}\")\n          end\n        end)\n      end\n\n      # Option 2: Define a helper function for reusability\n      defp handle_webhook(conn) do\n        case TopggEx.Webhook.verify_and_parse(conn, \"your_webhook_auth_token\") do\n          {:ok, payload} ->\n            case payload do\n              %{\"user\" => user_id, \"type\" => \"upvote\", \"bot\" => bot_id} ->\n                MyApp.handle_user_vote(user_id, bot_id)\n                IO.puts(\"User #{user_id} voted for bot #{bot_id}!\")\n\n              %{\"user\" => user_id, \"type\" => \"test\"} ->\n                IO.puts(\"Test webhook from user #{user_id}\")\n            end\n            send_resp(conn, 204, \"\")\n\n          {:error, reason} ->\n            send_resp(conn, 400, \"Error: #{inspect(reason)}\")\n        end\n      end\n\n      post \"/webhook-custom\" do\n        handle_webhook(conn)\n      end\n\n      match _ do\n        send_resp(conn, 404, \"Not found\")\n      end\n    end","ref":"TopggEx.Webhook.html#module-using-the-functional-listener-recommended"},{"type":"module","title":"Using as Plug Middleware (Alternative) - TopggEx.Webhook","doc":"# In your Phoenix router:\n    defmodule MyAppWeb.Router do\n      use MyAppWeb, :router\n\n      pipeline :webhook do\n        plug :accepts, [\"json\"]\n        plug TopggEx.Webhook, authorization: \"your_webhook_auth_token\"\n      end\n\n      scope \"/webhooks\" do\n        pipe_through :webhook\n        post \"/topgg\", YourController, :handle_vote\n      end\n    end\n\n    # In your controller:\n    defmodule YourController do\n      use MyAppWeb, :controller\n\n      def handle_vote(conn, _params) do\n        case conn.assigns.topgg_payload do\n          %{\"user\" => user_id, \"type\" => \"upvote\"} ->\n            # Handle the vote\n            IO.puts(\"User #{user_id} voted!\")\n            send_resp(conn, 204, \"\")\n\n          %{\"user\" => user_id, \"type\" => \"test\"} ->\n            # Handle test webhook\n            IO.puts(\"Test webhook from user #{user_id}\")\n            send_resp(conn, 204, \"\")\n\n          _ ->\n            send_resp(conn, 400, \"Invalid payload\")\n        end\n      end\n    end","ref":"TopggEx.Webhook.html#module-using-as-plug-middleware-alternative"},{"type":"module","title":"Functional API - TopggEx.Webhook","doc":"You can also use the functional API for more control:\n\n    # Verify and parse a webhook request\n    case TopggEx.Webhook.verify_and_parse(conn, \"your_auth_token\") do\n      {:ok, payload} ->\n        # Handle the vote payload\n        IO.inspect(payload)\n\n      {:error, :unauthorized} ->\n        # Handle unauthorized request\n        send_resp(conn, 403, Jason.encode!(%{error: \"Unauthorized\"}))\n\n      {:error, :invalid_body} ->\n        # Handle malformed request\n        send_resp(conn, 400, Jason.encode!(%{error: \"Invalid body\"}))\n    end","ref":"TopggEx.Webhook.html#module-functional-api"},{"type":"module","title":"Webhook Data Schema - TopggEx.Webhook","doc":"The webhook payload typically contains:\n- `user` - The ID of the user who voted\n- `type` - The type of vote (\"upvote\" or \"test\")\n- `bot` - The ID of the bot that was voted for\n- `isWeekend` - Whether the vote was cast on a weekend (counts as 2 votes)\n- `query` - Query parameters from the vote page (if any)","ref":"TopggEx.Webhook.html#module-webhook-data-schema"},{"type":"module","title":"Links - TopggEx.Webhook","doc":"- [Top.gg Webhook Documentation](https://docs.top.gg/resources/webhooks/)\n- [Top.gg Webhook Schema](https://docs.top.gg/resources/webhooks/#schema)","ref":"TopggEx.Webhook.html#module-links"},{"type":"function","title":"TopggEx.Webhook.handle_webhook/3","doc":"Handles a webhook request in standalone Plug applications.\n\nThis is a convenience function for standalone Plug applications that want\na simple way to handle webhook requests without defining their own helper functions.","ref":"TopggEx.Webhook.html#handle_webhook/3"},{"type":"function","title":"Parameters - TopggEx.Webhook.handle_webhook/3","doc":"* `conn` - The Plug connection\n  * `auth_token` - The expected authorization token (optional)\n  * `handler_fun` - Function that takes the payload as argument","ref":"TopggEx.Webhook.html#handle_webhook/3-parameters"},{"type":"function","title":"Returns - TopggEx.Webhook.handle_webhook/3","doc":"The connection with appropriate response sent.","ref":"TopggEx.Webhook.html#handle_webhook/3-returns"},{"type":"function","title":"Examples - TopggEx.Webhook.handle_webhook/3","doc":"# In standalone Plug router\n    post \"/webhook\" do\n      TopggEx.Webhook.handle_webhook(conn, \"my_auth_token\", fn payload ->\n        case payload do\n          %{\"user\" => user_id, \"type\" => \"upvote\", \"bot\" => bot_id} ->\n            MyApp.handle_user_vote(user_id, bot_id)\n            IO.puts(\"User #{user_id} voted for bot #{bot_id}!\")\n\n          %{\"user\" => user_id, \"type\" => \"test\"} ->\n            IO.puts(\"Test webhook from user #{user_id}\")\n        end\n      end)\n    end","ref":"TopggEx.Webhook.html#handle_webhook/3-examples"},{"type":"function","title":"TopggEx.Webhook.listener/2","doc":"Creates a functional webhook handler that can be used in controllers or other contexts.","ref":"TopggEx.Webhook.html#listener/2"},{"type":"function","title":"Parameters - TopggEx.Webhook.listener/2","doc":"* `handler_fun` - Function that takes the payload and connection\n  * `opts` - Options including `:authorization` and `:error_handler`","ref":"TopggEx.Webhook.html#listener/2-parameters"},{"type":"function","title":"Returns - TopggEx.Webhook.listener/2","doc":"A function that can be used as a Plug or called directly.","ref":"TopggEx.Webhook.html#listener/2-returns"},{"type":"function","title":"Examples - TopggEx.Webhook.listener/2","doc":"webhook_handler = TopggEx.Webhook.listener(fn payload, conn ->\n      case payload do\n        %{\"user\" => user_id, \"type\" => \"upvote\", \"bot\" => bot_id} ->\n          MyApp.handle_user_vote(user_id, bot_id)\n          IO.puts(\"User #{user_id} voted for bot #{bot_id}!\")\n\n        %{\"user\" => user_id, \"type\" => \"test\"} ->\n          IO.puts(\"Test webhook from user #{user_id}\")\n      end\n\n      # Response is handled automatically\n    end, authorization: \"my_auth_token\")\n\n    # Use in Phoenix router\n    post \"/webhook\", webhook_handler\n\n    # Use in standalone Plug router with helper function\n    defp handle_webhook(conn) do\n      case TopggEx.Webhook.verify_and_parse(conn, \"my_auth_token\") do\n        {:ok, payload} ->\n          # Handle payload...\n          send_resp(conn, 204, \"\")\n        {:error, reason} ->\n          send_resp(conn, 400, \"Error\")\n      end\n    end\n\n    post \"/webhook\" do\n      handle_webhook(conn)\n    end\n\n    # Or use verify_and_parse directly inline\n    post \"/webhook\" do\n      case TopggEx.Webhook.verify_and_parse(conn, \"my_auth_token\") do\n        {:ok, payload} ->\n          # Handle payload...\n          send_resp(conn, 204, \"\")\n        {:error, reason} ->\n          send_resp(conn, 400, \"Error\")\n      end\n    end","ref":"TopggEx.Webhook.html#listener/2-examples"},{"type":"function","title":"TopggEx.Webhook.verify_and_parse/2","doc":"Verifies the authorization header and parses the webhook payload.","ref":"TopggEx.Webhook.html#verify_and_parse/2"},{"type":"function","title":"Parameters - TopggEx.Webhook.verify_and_parse/2","doc":"* `conn` - The Plug connection\n  * `expected_auth` - The expected authorization token (optional)","ref":"TopggEx.Webhook.html#verify_and_parse/2-parameters"},{"type":"function","title":"Returns - TopggEx.Webhook.verify_and_parse/2","doc":"* `{:ok, payload}` - Successfully parsed and validated webhook payload\n  * `{:error, :unauthorized}` - Authorization header doesn't match\n  * `{:error, :invalid_body}` - Request body is not valid JSON\n  * `{:error, :malformed_request}` - Error reading request body\n  * `{:error, :invalid_payload_format}` - Payload is not a map\n  * `{:error, {:missing_fields, fields}}` - Required fields are missing\n  * `{:error, {:invalid_field_type, field}}` - Field has incorrect type","ref":"TopggEx.Webhook.html#verify_and_parse/2-returns"},{"type":"function","title":"Examples - TopggEx.Webhook.verify_and_parse/2","doc":"case TopggEx.Webhook.verify_and_parse(conn, \"my_auth_token\") do\n      {:ok, payload} ->\n        case payload do\n          %{\"user\" => user_id, \"type\" => \"upvote\", \"bot\" => bot_id} ->\n            IO.puts(\"Received vote from user: #{user_id} for bot: #{bot_id}\")\n            send_resp(conn, 204, \"\")\n\n          %{\"user\" => user_id, \"type\" => \"test\"} ->\n            IO.puts(\"Test webhook from user: #{user_id}\")\n            send_resp(conn, 204, \"\")\n        end\n\n      {:error, reason} ->\n        IO.puts(\"Webhook error: #{inspect(reason)}\")\n        send_resp(conn, 400, \"Error\")\n    end","ref":"TopggEx.Webhook.html#verify_and_parse/2-examples"},{"type":"type","title":"TopggEx.Webhook.webhook_options/0","doc":"","ref":"TopggEx.Webhook.html#t:webhook_options/0"},{"type":"type","title":"TopggEx.Webhook.webhook_payload/0","doc":"","ref":"TopggEx.Webhook.html#t:webhook_payload/0"},{"type":"module","title":"TopggEx.Api","doc":"Top.gg API Client for posting stats and fetching data.\n\nThis module provides a comprehensive client for interacting with the Top.gg API,\nallowing you to post bot statistics, retrieve bot information, check votes, and more.","ref":"TopggEx.Api.html"},{"type":"module","title":"Examples - TopggEx.Api","doc":"iex> {:ok, api} = TopggEx.Api.new(\"your_topgg_token_here\")\n    iex> {:ok, _stats} = TopggEx.Api.post_stats(api, %{server_count: 100})\n    {:ok, %{server_count: 100}}","ref":"TopggEx.Api.html#module-examples"},{"type":"module","title":"Configuration - TopggEx.Api","doc":"Before using this module, ensure you have a Finch instance running in your\napplication supervision tree:\n\n    children = [\n      {Finch, name: :topgg_finch}\n    ]","ref":"TopggEx.Api.html#module-configuration"},{"type":"module","title":"See also - TopggEx.Api","doc":"- [Top.gg API Documentation](https://docs.top.gg)\n- [Top.gg Library Documentation](https://topgg.js.org)","ref":"TopggEx.Api.html#module-see-also"},{"type":"type","title":"TopggEx.Api.api_options/0","doc":"","ref":"TopggEx.Api.html#t:api_options/0"},{"type":"type","title":"TopggEx.Api.bot_info/0","doc":"","ref":"TopggEx.Api.html#t:bot_info/0"},{"type":"type","title":"TopggEx.Api.bot_stats_response/0","doc":"","ref":"TopggEx.Api.html#t:bot_stats_response/0"},{"type":"type","title":"TopggEx.Api.bots_query/0","doc":"","ref":"TopggEx.Api.html#t:bots_query/0"},{"type":"type","title":"TopggEx.Api.bots_response/0","doc":"","ref":"TopggEx.Api.html#t:bots_response/0"},{"type":"function","title":"TopggEx.Api.get_bot/2","doc":"Gets bot information from Top.gg.\n\nReturns `{:ok, bot_info}` on success, or `{:error, reason}` on failure.","ref":"TopggEx.Api.html#get_bot/2"},{"type":"function","title":"Parameters - TopggEx.Api.get_bot/2","doc":"* `api` - API client instance\n  * `id` - Bot ID (snowflake)","ref":"TopggEx.Api.html#get_bot/2-parameters"},{"type":"function","title":"Examples - TopggEx.Api.get_bot/2","doc":"iex> {:ok, bot} = TopggEx.Api.get_bot(api, \"461521980492087297\")\n    {:ok, %{\"id\" => \"461521980492087297\", \"username\" => \"Shiro\", ...}}","ref":"TopggEx.Api.html#get_bot/2-examples"},{"type":"function","title":"TopggEx.Api.get_bots/2","doc":"Gets a list of bots from Top.gg.\n\nReturns `{:ok, bots_response}` on success, or `{:error, reason}` on failure.","ref":"TopggEx.Api.html#get_bots/2"},{"type":"function","title":"Parameters - TopggEx.Api.get_bots/2","doc":"* `api` - API client instance\n  * `query` - Optional query parameters","ref":"TopggEx.Api.html#get_bots/2-parameters"},{"type":"function","title":"Examples - TopggEx.Api.get_bots/2","doc":"# Finding by properties\n    iex> {:ok, response} = TopggEx.Api.get_bots(api, %{\n    ...>   search: %{username: \"shiro\"}\n    ...> })\n    {:ok, %{results: [%{\"id\" => \"461521980492087297\", \"username\" => \"Shiro\", ...}], ...}}\n\n    # Restricting fields\n    iex> {:ok, response} = TopggEx.Api.get_bots(api, %{\n    ...>   fields: [\"id\", \"username\"]\n    ...> })\n    {:ok, %{results: [%{\"id\" => \"461521980492087297\", \"username\" => \"Shiro\"}], ...}}","ref":"TopggEx.Api.html#get_bots/2-examples"},{"type":"function","title":"TopggEx.Api.get_stats/1","doc":"Gets your bot's statistics from Top.gg.\n\nReturns `{:ok, stats}` on success, or `{:error, reason}` on failure.","ref":"TopggEx.Api.html#get_stats/1"},{"type":"function","title":"Parameters - TopggEx.Api.get_stats/1","doc":"* `api` - API client instance","ref":"TopggEx.Api.html#get_stats/1-parameters"},{"type":"function","title":"Examples - TopggEx.Api.get_stats/1","doc":"iex> {:ok, stats} = TopggEx.Api.get_stats(api)\n    {:ok, %{server_count: 28199, shard_count: 5, shards: [5000, 5000, 5000, 5000, 8199]}}","ref":"TopggEx.Api.html#get_stats/1-examples"},{"type":"function","title":"TopggEx.Api.get_user/2","doc":"Gets user information from Top.gg.\n\nReturns `{:ok, user_info}` on success, or `{:error, reason}` on failure.\n\n> #### Deprecated {: .warning}\n>\n> This function is deprecated and no longer supported by Top.gg API v0.","ref":"TopggEx.Api.html#get_user/2"},{"type":"function","title":"Parameters - TopggEx.Api.get_user/2","doc":"* `api` - API client instance\n  * `id` - User ID (snowflake)","ref":"TopggEx.Api.html#get_user/2-parameters"},{"type":"function","title":"Examples - TopggEx.Api.get_user/2","doc":"iex> {:ok, user} = TopggEx.Api.get_user(api, \"205680187394752512\")\n    {:ok, %{\"id\" => \"205680187394752512\", \"username\" => \"Xignotic\", ...}}","ref":"TopggEx.Api.html#get_user/2-examples"},{"type":"function","title":"TopggEx.Api.get_votes/2","doc":"Gets recent unique users who have voted for your bot.\n\nReturns `{:ok, [users]}` on success, or `{:error, reason}` on failure.","ref":"TopggEx.Api.html#get_votes/2"},{"type":"function","title":"Parameters - TopggEx.Api.get_votes/2","doc":"* `api` - API client instance\n  * `page` - Optional page number (each page has at most 100 voters)","ref":"TopggEx.Api.html#get_votes/2-parameters"},{"type":"function","title":"Examples - TopggEx.Api.get_votes/2","doc":"iex> {:ok, voters} = TopggEx.Api.get_votes(api)\n    {:ok, [%{username: \"Xignotic\", id: \"205680187394752512\", avatar: \"https://...\"}, ...]}\n\n    iex> {:ok, voters} = TopggEx.Api.get_votes(api, 2)\n    {:ok, [%{username: \"iara\", id: \"395526710101278721\", avatar: \"https://...\"}, ...]}","ref":"TopggEx.Api.html#get_votes/2-examples"},{"type":"function","title":"TopggEx.Api.has_voted/2","doc":"Checks whether or not a user has voted in the last 12 hours.\n\nReturns `{:ok, boolean}` on success, or `{:error, reason}` on failure.","ref":"TopggEx.Api.html#has_voted/2"},{"type":"function","title":"Parameters - TopggEx.Api.has_voted/2","doc":"* `api` - API client instance\n  * `id` - User ID (snowflake)","ref":"TopggEx.Api.html#has_voted/2-parameters"},{"type":"function","title":"Examples - TopggEx.Api.has_voted/2","doc":"iex> {:ok, voted?} = TopggEx.Api.has_voted(api, \"205680187394752512\")\n    {:ok, true}","ref":"TopggEx.Api.html#has_voted/2-examples"},{"type":"function","title":"TopggEx.Api.is_weekend/1","doc":"Checks whether or not the weekend multiplier is active.\n\nReturns `{:ok, boolean}` on success, or `{:error, reason}` on failure.","ref":"TopggEx.Api.html#is_weekend/1"},{"type":"function","title":"Parameters - TopggEx.Api.is_weekend/1","doc":"* `api` - API client instance","ref":"TopggEx.Api.html#is_weekend/1-parameters"},{"type":"function","title":"Examples - TopggEx.Api.is_weekend/1","doc":"iex> {:ok, is_weekend?} = TopggEx.Api.is_weekend(api)\n    {:ok, false}","ref":"TopggEx.Api.html#is_weekend/1-examples"},{"type":"function","title":"TopggEx.Api.new/2","doc":"Creates a new Top.gg API client instance.\n\nReturns `:ok` with the API client struct, or `{:error, reason}` if the token is malformed.","ref":"TopggEx.Api.html#new/2"},{"type":"function","title":"Parameters - TopggEx.Api.new/2","doc":"* `token` - Your Top.gg API token\n  * `options` - Optional configuration map","ref":"TopggEx.Api.html#new/2-parameters"},{"type":"function","title":"Options - TopggEx.Api.new/2","doc":"* `:finch_name` - Name of the Finch instance to use (default: `:topgg_finch`)\n  * `:base_url` - Base URL for the API (default: `\"https://top.gg/api\"`)","ref":"TopggEx.Api.html#new/2-options"},{"type":"function","title":"Examples - TopggEx.Api.new/2","doc":"iex> {:ok, api} = TopggEx.Api.new(\"your_token_here\")\n    iex> {:ok, api} = TopggEx.Api.new(\"your_token_here\", %{finch_name: :my_finch})","ref":"TopggEx.Api.html#new/2-examples"},{"type":"function","title":"TopggEx.Api.post_stats/2","doc":"Posts bot statistics to Top.gg.\n\nReturns `{:ok, stats}` on success, or `{:error, reason}` if the server count\nis missing, invalid, or the API request fails.","ref":"TopggEx.Api.html#post_stats/2"},{"type":"function","title":"Parameters - TopggEx.Api.post_stats/2","doc":"* `api` - API client instance\n  * `stats` - Stats map containing at least `:server_count`","ref":"TopggEx.Api.html#post_stats/2-parameters"},{"type":"function","title":"Examples - TopggEx.Api.post_stats/2","doc":"# Posting server count as an integer\n    iex> {:ok, stats} = TopggEx.Api.post_stats(api, %{server_count: 28199})\n    {:ok, %{server_count: 28199}}\n\n    # Posting with shards (server_count as array)\n    iex> {:ok, stats} = TopggEx.Api.post_stats(api, %{server_count: [1000, 2000, 3000]})\n    {:ok, %{server_count: [1000, 2000, 3000]}}\n\n    # Posting with additional shard information\n    iex> {:ok, stats} = TopggEx.Api.post_stats(api, %{\n    ...>   server_count: 500,\n    ...>   shard_id: 0,\n    ...>   shard_count: 5\n    ...> })\n    {:ok, %{server_count: 500, shard_id: 0, shard_count: 5}}","ref":"TopggEx.Api.html#post_stats/2-examples"},{"type":"type","title":"TopggEx.Api.post_stats_body/0","doc":"","ref":"TopggEx.Api.html#t:post_stats_body/0"},{"type":"type","title":"TopggEx.Api.short_user/0","doc":"","ref":"TopggEx.Api.html#t:short_user/0"},{"type":"type","title":"TopggEx.Api.snowflake/0","doc":"","ref":"TopggEx.Api.html#t:snowflake/0"},{"type":"type","title":"TopggEx.Api.t/0","doc":"","ref":"TopggEx.Api.html#t:t/0"},{"type":"type","title":"TopggEx.Api.user_info/0","doc":"","ref":"TopggEx.Api.html#t:user_info/0"},{"type":"module","title":"TopggEx.HttpClient","doc":"HTTP client for making requests to the Top.gg API using Finch.\n\nThis module handles all HTTP communication with the Top.gg API,\nincluding request building, execution, and response parsing.","ref":"TopggEx.HttpClient.html"},{"type":"function","title":"TopggEx.HttpClient.request/4","doc":"Makes an HTTP request to the Top.gg API.","ref":"TopggEx.HttpClient.html#request/4"},{"type":"function","title":"Parameters - TopggEx.HttpClient.request/4","doc":"* `options` - Request configuration containing token, finch_name, and base_url\n  * `method` - HTTP method (:get, :post, etc.)\n  * `path` - API endpoint path\n  * `body` - Request body (optional)","ref":"TopggEx.HttpClient.html#request/4-parameters"},{"type":"function","title":"Examples - TopggEx.HttpClient.request/4","doc":"iex> options = %{token: \"token\", finch_name: :topgg_finch, base_url: \"https://top.gg/api\"}\n    iex> TopggEx.HttpClient.request(options, :get, \"/bots/stats\")\n    {:ok, %{\"server_count\" => 100}}","ref":"TopggEx.HttpClient.html#request/4-examples"},{"type":"type","title":"TopggEx.HttpClient.request_options/0","doc":"","ref":"TopggEx.HttpClient.html#t:request_options/0"},{"type":"behaviour","title":"TopggEx.HttpClientBehaviour","doc":"Behaviour for HTTP client implementations.\n\nThis behaviour defines the contract for HTTP clients used by the TopggEx API.\nIt enables dependency injection and mocking for testing purposes.","ref":"TopggEx.HttpClientBehaviour.html"},{"type":"callback","title":"TopggEx.HttpClientBehaviour.request/4","doc":"","ref":"TopggEx.HttpClientBehaviour.html#c:request/4"},{"type":"type","title":"TopggEx.HttpClientBehaviour.request_options/0","doc":"","ref":"TopggEx.HttpClientBehaviour.html#t:request_options/0"},{"type":"extras","title":"TopggEx","doc":"# TopggEx\n\n[![Hex.pm](https://img.shields.io/hexpm/v/topgg_ex.svg)](https://hex.pm/packages/topgg_ex)\n[![Documentation](https://img.shields.io/badge/docs-hexdocs-blue.svg)](https://hexdocs.pm/topgg_ex/)\n[![License](https://img.shields.io/hexpm/l/topgg_ex.svg)](https://github.com/jilv220/topgg_ex/blob/master/LICENSE)\n\nA community Elixir SDK for the [Top.gg](https://top.gg) API, allowing you to interact with Discord bot statistics, user votes, and bot information.","ref":"readme.html"},{"type":"extras","title":"Features - TopggEx","doc":"- 🚀 **Complete API Coverage**: All Top.gg API endpoints supported\n- 🎯 **Webhook Support**: Built-in webhook handler for vote notifications\n- 🔒 **Type Safety**: Full typespecs and structured data\n- ⚡ **HTTP/2 Support**: Built on Finch for modern HTTP performance\n- 🧪 **Well Tested**: Comprehensive test suite with 95%+ coverage\n- 📚 **Excellent Documentation**: Detailed docs with examples\n- 🏗️ **Clean Architecture**: Separated HTTP client for maintainability","ref":"readme.html#features"},{"type":"extras","title":"Installation - TopggEx","doc":"Add `topgg_ex` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:topgg_ex, \"~> 0.1.0\"},\n    {:finch, \"~> 0.19\"}  # Required HTTP client\n  ]\nend\n```\n\nThen run:\n\n```bash\nmix deps.get\n```","ref":"readme.html#installation"},{"type":"extras","title":"Quick Start - TopggEx","doc":"","ref":"readme.html#quick-start"},{"type":"extras","title":"1. Setup - TopggEx","doc":"First, add Finch to your application's supervision tree:\n\n```elixir\n# In your application.ex\nchildren = [\n  {Finch, name: :topgg_finch}\n]\n```","ref":"readme.html#1-setup"},{"type":"extras","title":"2. Create API Client - TopggEx","doc":"```elixir\n# Get your token from https://top.gg/api/docs#mybots\n{:ok, api} = TopggEx.Api.new(\"your_topgg_token_here\")\n```","ref":"readme.html#2-create-api-client"},{"type":"extras","title":"3. Post Bot Statistics - TopggEx","doc":"```elixir\n# Update your bot's server count\n{:ok, stats} = TopggEx.Api.post_stats(api, %{server_count: 1250})\n```","ref":"readme.html#3-post-bot-statistics"},{"type":"extras","title":"Usage Examples - TopggEx","doc":"","ref":"readme.html#usage-examples"},{"type":"extras","title":"Bot Statistics - TopggEx","doc":"```elixir\n# Post bot stats\n{:ok, _} = TopggEx.Api.post_stats(api, %{\n  server_count: 1250,\n  shard_count: 2,\n  shards: [625, 625]\n})\n\n# Get your bot's current stats\n{:ok, stats} = TopggEx.Api.get_stats(api)\n# => %{server_count: 1250, shard_count: 2, shards: [625, 625]}\n```","ref":"readme.html#bot-statistics"},{"type":"extras","title":"Bot Information - TopggEx","doc":"```elixir\n# Get information about any bot\n{:ok, bot} = TopggEx.Api.get_bot(api, \"461521980492087297\")\n# => %{\"id\" => \"461521980492087297\", \"username\" => \"Shiro\", ...}\n\n# Search for bots\n{:ok, results} = TopggEx.Api.get_bots(api, %{\n  search: %{username: \"music\"},\n  limit: 10,\n  fields: [\"id\", \"username\", \"short_description\"]\n})\n```","ref":"readme.html#bot-information"},{"type":"extras","title":"Vote Checking - TopggEx","doc":"```elixir\n# Check if a user has voted\n{:ok, has_voted?} = TopggEx.Api.has_voted(api, \"205680187394752512\")\n# => true or false\n\n# Get recent voters\n{:ok, voters} = TopggEx.Api.get_votes(api)\n# => [%{\"username\" => \"Example\", \"id\" => \"123...\", \"avatar\" => \"https://...\"}, ...]\n\n# Check weekend multiplier status\n{:ok, is_weekend?} = TopggEx.Api.is_weekend(api)\n# => true or false\n```","ref":"readme.html#vote-checking"},{"type":"extras","title":"Webhook Handling - TopggEx","doc":"TopggEx includes a built-in webhook handler for receiving vote notifications from Top.gg.\n\n#### Using the Functional Listener (Recommended)\n\nThe easiest way to handle webhooks is using the `listener` function:\n\n```elixir\n# In your Phoenix router\ndefmodule MyAppWeb.Router do\n  use MyAppWeb, :router\n\n  # Create the webhook handler\n  webhook_handler = TopggEx.Webhook.listener(fn payload, conn ->\n    case payload do\n      %{\"user\" => user_id, \"type\" => \"upvote\", \"bot\" => bot_id} ->\n        # Handle the vote\n        MyApp.handle_user_vote(user_id, bot_id)\n        IO.puts(\"User #{user_id} voted for bot #{bot_id}!\")\n\n      %{\"user\" => user_id, \"type\" => \"test\"} ->\n        # Handle test webhook\n        IO.puts(\"Test webhook from user: #{user_id}\")\n    end\n\n    # Response is handled automatically by the listener\n  end, authorization: \"your_webhook_auth_token\")\n\n  scope \"/webhooks\" do\n    pipe_through :api\n    post \"/topgg\", webhook_handler\n  end\nend\n```\n\n**For standalone Plug applications:**\n\n```elixir\n# Define your router module\ndefmodule MyApp.Router do\n  use Plug.Router\n\n  plug :match\n  plug :dispatch\n\n  # Option 1: Use the convenient handle_webhook function (Recommended)\n  post \"/webhook\" do\n    TopggEx.Webhook.handle_webhook(conn, \"your_webhook_auth_token\", fn payload ->\n      case payload do\n        %{\"user\" => user_id, \"type\" => \"upvote\", \"bot\" => bot_id} ->\n          # Handle the vote\n          MyApp.handle_user_vote(user_id, bot_id)\n          IO.puts(\"User #{user_id} voted for bot #{bot_id}!\")\n\n        %{\"user\" => user_id, \"type\" => \"test\"} ->\n          # Handle test webhook\n          IO.puts(\"Test webhook from user: #{user_id}\")\n      end\n    end)\n  end\n\n  # Option 2: Use verify_and_parse directly (more explicit)\n  post \"/webhook-direct\" do\n    case TopggEx.Webhook.verify_and_parse(conn, \"your_webhook_auth_token\") do\n      {:ok, payload} ->\n        case payload do\n          %{\"user\" => user_id, \"type\" => \"upvote\", \"bot\" => bot_id} ->\n            MyApp.handle_user_vote(user_id, bot_id)\n            IO.puts(\"User #{user_id} voted for bot #{bot_id}!\")\n\n          %{\"user\" => user_id, \"type\" => \"test\"} ->\n            IO.puts(\"Test webhook from user: #{user_id}\")\n        end\n        send_resp(conn, 204, \"\")\n\n      {:error, reason} ->\n        send_resp(conn, 400, \"Webhook error: #{inspect(reason)}\")\n    end\n  end\n\n  match _ do\n    send_resp(conn, 404, \"Not found\")\n  end\nend\n\n# Start your application\n{:ok, _} = Plug.Cowboy.http(MyApp.Router, [])\n```\n\n#### Using as Plug Middleware (Alternative)\n\nYou can also use the webhook handler as Plug middleware:\n\n```elixir\n# In your Phoenix router\ndefmodule MyAppWeb.Router do\n  use MyAppWeb, :router\n\n  pipeline :webhook do\n    plug :accepts, [\"json\"]\n    plug TopggEx.Webhook, authorization: \"your_webhook_auth_token\"\n  end\n\n  scope \"/webhooks\" do\n    pipe_through :webhook\n    post \"/topgg\", MyAppWeb.WebhookController, :handle_vote\n  end\nend\n\n# In your controller\ndefmodule MyAppWeb.WebhookController do\n  use MyAppWeb, :controller\n\n  def handle_vote(conn, _params) do\n    case conn.assigns.topgg_payload do\n      %{\"user\" => user_id, \"type\" => \"upvote\", \"bot\" => bot_id} ->\n        # Handle the vote\n        MyApp.handle_user_vote(user_id, bot_id)\n        send_resp(conn, 204, \"\")\n\n      %{\"user\" => user_id, \"type\" => \"test\"} ->\n        # Handle test webhook\n        IO.puts(\"Test webhook from user: #{user_id}\")\n        send_resp(conn, 204, \"\")\n    end\n  end\nend\n```\n\n#### Manual Webhook Processing\n\nFor maximum control, you can use the functional API to manually parse webhooks:\n\n```elixir\ndef handle_webhook(conn) do\n  case TopggEx.Webhook.verify_and_parse(conn, \"your_auth_token\") do\n    {:ok, payload} ->\n      case payload do\n        %{\"user\" => user_id, \"type\" => \"upvote\", \"bot\" => bot_id} ->\n          MyApp.process_vote(user_id, bot_id)\n          send_resp(conn, 204, \"\")\n\n        %{\"user\" => user_id, \"type\" => \"test\"} ->\n          IO.puts(\"Test webhook from user: #{user_id}\")\n          send_resp(conn, 204, \"\")\n\n\n      end\n\n    {:error, :unauthorized} ->\n      send_resp(conn, 403, Jason.encode!(%{error: \"Unauthorized\"}))\n\n    {:error, :invalid_body} ->\n      send_resp(conn, 400, Jason.encode!(%{error: \"Invalid body\"}))\n\n    {:error, reason} ->\n      send_resp(conn, 400, Jason.encode!(%{error: \"Webhook error: #{inspect(reason)}\"}))\n  end\nend\n```\n\n#### Advanced Listener Usage\n\nYou can create more sophisticated webhook handlers with error handling and custom logic:\n\n```elixir\nwebhook_handler = TopggEx.Webhook.listener(fn payload, conn ->\n  %{\"user\" => user_id, \"type\" => vote_type, \"bot\" => bot_id} = payload\n\n  case vote_type do\n    \"upvote\" ->\n      # Record the vote and handle weekend multiplier\n      is_weekend = Map.get(payload, \"isWeekend\", false)\n      vote_count = if is_weekend, do: 2, else: 1\n\n      MyApp.record_vote(user_id, bot_id, vote_count)\n      MyApp.send_thank_you(user_id)\n\n      IO.puts(\"User #{user_id} voted! (#{vote_count} votes)\")\n\n    \"test\" ->\n      IO.puts(\"Test webhook received from user #{user_id}!\")\n\n\n  end\n\n  # Response is handled automatically by the listener\n  # No need to call send_resp/3\nend, authorization: \"your_webhook_auth_token\")\n\n# Use in router\npost \"/webhook\", webhook_handler\n```","ref":"readme.html#webhook-handling"},{"type":"extras","title":"Advanced Usage - TopggEx","doc":"```elixir\n# Custom Finch instance\n{:ok, api} = TopggEx.Api.new(\"your_token\", %{\n  finch_name: :my_custom_finch,\n  base_url: \"https://top.gg/api\"  # Optional custom base URL\n})\n\n# Complex bot search\n{:ok, results} = TopggEx.Api.get_bots(api, %{\n  search: %{\n    username: \"music bot\",\n    tags: \"music\"\n  },\n  sort: \"server_count\",\n  limit: 50,\n  fields: [\"id\", \"username\", \"short_description\", \"server_count\"]\n})\n```","ref":"readme.html#advanced-usage"},{"type":"extras","title":"API Reference - TopggEx","doc":"","ref":"readme.html#api-reference"},{"type":"extras","title":"Core Functions - TopggEx","doc":"#### API Client (`TopggEx.Api`)\n\n| Function       | Description              | Parameters         |\n| -------------- | ------------------------ | ------------------ |\n| `new/2`        | Create API client        | `token`, `options` |\n| `post_stats/2` | Update bot statistics    | `api`, `stats`     |\n| `get_stats/1`  | Get bot statistics       | `api`              |\n| `get_bot/2`    | Get bot information      | `api`, `bot_id`    |\n| `get_bots/2`   | Search bots              | `api`, `query`     |\n| `get_votes/2`  | Get recent voters        | `api`, `page`      |\n| `has_voted/2`  | Check user vote status   | `api`, `user_id`   |\n| `is_weekend/1` | Check weekend multiplier | `api`              |\n\n#### Webhook Handler (`TopggEx.Webhook`)\n\n| Function             | Description                    | Parameters              |\n| -------------------- | ------------------------------ | ----------------------- |\n| `verify_and_parse/2` | Parse webhook payload          | `conn`, `auth_token`    |\n| `listener/2`         | Create functional handler      | `handler_fun`, `opts`   |\n| Plug behavior        | Use as Phoenix/Plug middleware | `authorization`, `opts` |","ref":"readme.html#core-functions"},{"type":"extras","title":"Error Handling - TopggEx","doc":"All functions return `{:ok, result}` on success or `{:error, reason}` on failure:\n\n```elixir\ncase TopggEx.Api.post_stats(api, %{server_count: 100}) do\n  {:ok, stats} ->\n    IO.puts(\"Stats updated successfully!\")\n  {:error, %{status: 401}} ->\n    IO.puts(\"Invalid API token\")\n  {:error, %{status: 429}} ->\n    IO.puts(\"Rate limited - try again later\")\n  {:error, reason} ->\n    IO.puts(\"Network error: #{inspect(reason)}\")\nend\n```","ref":"readme.html#error-handling"},{"type":"extras","title":"Configuration - TopggEx","doc":"","ref":"readme.html#configuration"},{"type":"extras","title":"Environment Variables - TopggEx","doc":"You can set your Top.gg token via environment variables:\n\n```elixir\n# config/runtime.exs\nconfig :my_app, :topgg_token, System.get_env(\"TOPGG_TOKEN\")\n\n# In your application\ntoken = Application.get_env(:my_app, :topgg_token)\n{:ok, api} = TopggEx.Api.new(token)\n```","ref":"readme.html#environment-variables"},{"type":"extras","title":"Custom HTTP Client Options - TopggEx","doc":"```elixir\n{:ok, api} = TopggEx.Api.new(\"your_token\", %{\n  finch_name: :my_finch,      # Custom Finch instance name\n  base_url: \"https://top.gg/api\"  # Custom API base URL\n})\n```","ref":"readme.html#custom-http-client-options"},{"type":"extras","title":"Rate Limiting - TopggEx","doc":"Top.gg API has rate limits. The library will return appropriate errors:\n\n- **429 Too Many Requests**: You've hit the rate limit\n- **403 Forbidden**: Invalid token or insufficient permissions\n\nImplement exponential backoff for production applications:\n\n```elixir\ndefmodule MyBot.Stats do\n  def update_stats_with_retry(api, stats, retries \\\\ 3) do\n    case TopggEx.Api.post_stats(api, stats) do\n      {:ok, result} -> {:ok, result}\n      {:error, %{status: 429}} when retries > 0 ->\n        Process.sleep(1000 * (4 - retries))  # Exponential backoff\n        update_stats_with_retry(api, stats, retries - 1)\n      error -> error\n    end\n  end\nend\n```","ref":"readme.html#rate-limiting"},{"type":"extras","title":"Testing - TopggEx","doc":"The library includes comprehensive tests. Run them with:\n\n```bash\nmix test\n```\n\nFor testing your own applications, you can mock the HTTP client:\n\n```elixir\n# In your tests\nsetup do\n  bypass = Bypass.open()\n\n  {:ok, api} = TopggEx.Api.new(\"test_token\", %{\n    base_url: \"http://localhost:#{bypass.port}/api\"\n  })\n\n  {:ok, %{bypass: bypass, api: api}}\nend\n```","ref":"readme.html#testing"},{"type":"extras","title":"Contributing - TopggEx","doc":"1. Fork the repository\n2. Create your feature branch (`git checkout -b my-new-feature`)\n3. Commit your changes (`git commit -am 'Add some feature'`)\n4. Push to the branch (`git push origin my-new-feature`)\n5. Create a new Pull Request","ref":"readme.html#contributing"},{"type":"extras","title":"Development - TopggEx","doc":"```bash\n# Get dependencies\nmix deps.get\n\n# Run tests\nmix test\n\n# Generate documentation\nmix docs\n\n# Check formatting\nmix format --check-formatted\n\n# Run static analysis\nmix dialyzer\n```","ref":"readme.html#development"},{"type":"extras","title":"License - TopggEx","doc":"This project is licensed under the MIT License - see the [LICENSE](LICENSE) file for details.","ref":"readme.html#license"},{"type":"extras","title":"Links - TopggEx","doc":"- [Top.gg API Documentation](https://docs.top.gg)\n- [Hex Package](https://hex.pm/packages/topgg_ex)\n- [Documentation](https://hexdocs.pm/topgg_ex/)\n- [GitHub Repository](https://github.com/jilv220/topgg_ex)","ref":"readme.html#links"},{"type":"extras","title":"Acknowledgments - TopggEx","doc":"- Thanks to the [Top.gg](https://top.gg) team for providing the API\n- Built with [Finch](https://github.com/sneako/finch) for modern HTTP performance\n- Inspired by the JavaScript [topgg.js](https://github.com/top-gg/node-sdk) library","ref":"readme.html#acknowledgments"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright (c) 2024 TopggEx Contributors\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}